% photo3dPrint.m
% Mike Koehle 3/01/2016
% The user must set the folder containing the image, image name, the image width, and the top coat
% material (black or white). It is designed and tested for the Objet Connex
% 500, with tango black and vero white or vero black and vero white.. 

tic %set start time
clear
dir = 'C:\Users\mkoehle\Documents\ART\Relief\fromScratch\flatPhoto\finals\';
fileName = 'blackBay.jpg'; 
reliefWidth = 1; % desired width output in inches 
whiteTop = 0; % for black on top, set this to 1. For white on top, set to 0. 
baseThickness = .0; % thickness at base. Total height is reliefZ + baseThickness

baseOn = 0;

[pathstr,name,ext] = fileparts(fileName);
rawImage = imread([dir fileName]);


if length(size(rawImage))==3  %full color
    pixelVals = sum(rawImage,3)/3/255; %normalized pixel values
else % blackWhite Image
    pixelVals = double(rawImage)/255;
end

if whiteTop   
    pixelVals = fliplr(1-pixelVals); %%inverse image for white top/black base
else
     pixelVals = fliplr(pixelVals); 
end
pixelVals = imrotate(pixelVals,180);
imageSize = size(rawImage);

%attenuation adjustment:
if whiteTop  
    reliefZ = .12;  % max height of the relief, not including base thickness
    A = 90;
else % if black on top
    reliefZ = .04;
    A = 120;
end

I0 = 1;
pixelVals=(log(I0) - log(pixelVals))/A;
pixelVals(pixelVals>reliefZ)=reliefZ;


pixelVals = (1-pixelVals/reliefZ)*reliefZ;


xyScale = reliefWidth/(imageSize(2)-1);
yValues = ([1:imageSize(1)]-1)*xyScale; % linked to m; make a vector of values along the y axis, from 0 to picture height
xValues = ([1:imageSize(2)]-1)*xyScale; %linked to n
M=imageSize(1);
N = imageSize(2);

% Base STL---------------------
% this can be replaced by the box. 
if baseOn
    
    v=[];
    f=[];
    triCount = 0;
    verticeCount = 0;

    for m=1:imageSize(1)-1
        v1 =[];
        v2=[];
        for n=1:imageSize(2)
            v1 = [v1; xValues(n), yValues(m), pixelVals(m,n)+baseThickness];
            v2 = [v2; xValues(n), yValues(m+1), pixelVals(m+1,n)+baseThickness];
        end
        [fNew,vNew] = addQuadStrip(v1,v2);
        verticeCount = length(v);
        f = [f;fNew+verticeCount];
        v = [v;vNew];
    end

    % drawSides:
    % bottom edge

    for n=1:N-1
            pt1 = [xValues(n), 0, 0];
            pt2 = [xValues(n+1), 0, 0];
            pt3 = [xValues(n+1), 0, pixelVals(1,n+1)+baseThickness];
            pt4 = [xValues(n),0, pixelVals(1,n)+baseThickness];
            [fNew,vNew] = add2Triangles(pt1,pt2,pt3,pt4);
            verticeCount = length(v);
            v = [v;vNew];
            f = [f;fNew+verticeCount];

     end

    % right edge
    for m=1:M-1
            pt1 = [xValues(end), yValues(m), 0];
            pt2 = [xValues(end), yValues(m+1), 0];
            pt3 = [xValues(end), yValues(m+1), pixelVals(m+1,N)+baseThickness];
            pt4 = [xValues(end),yValues(m), pixelVals(m,N)+baseThickness];
            [fNew,vNew] = add2Triangles(pt1,pt2,pt3,pt4);
            verticeCount = length(v);
            v = [v;vNew];
            f = [f;fNew+verticeCount];

    end

    %top edge
    for n=N:-1:2

            pt1 = [xValues(n), yValues(end), 0];
            pt2 = [xValues(n-1), yValues(end), 0];
            pt3 = [xValues(n-1), yValues(end), pixelVals(M,n-1)+baseThickness];
            pt4 = [xValues(n), yValues(end), pixelVals(M,n)+baseThickness];
            [fNew,vNew] = add2Triangles(pt1,pt2,pt3,pt4);
            verticeCount = length(v);
            v = [v;vNew];
            f = [f;fNew+verticeCount];
    end

    %left edge:
    for m=M:-1:2
            pt1 = [0, yValues(m), 0];
            pt2 = [0, yValues(m-1), 0];
            pt3 = [0, yValues(m-1), pixelVals(m-1,1)+baseThickness];
            pt4 = [0, yValues(m), pixelVals(m,1)+baseThickness];
            [fNew,vNew] = add2Triangles(pt1,pt2,pt3,pt4);
            verticeCount = length(v);
            v = [v;vNew];
            f = [f;fNew+verticeCount];

    end

    % bottom surface, using fan function
    bottomEdge = [xValues' zeros(N,1) zeros(N,1)]; %bottom edge
    rightEdge = [xValues(end)*ones(M-1,1) yValues(2:M)' zeros(M-1,1)];
    topEdge = [xValues(N-1:-1:1)', yValues(end)*ones(N-1,1), zeros(N-1,1)];
    leftEdge = [zeros(M-2,1), yValues(M-1:-1:2)', zeros(M-2,1)];
    pts =[bottomEdge; rightEdge; topEdge; leftEdge];
    [fNew,vNew] = trianglesFanDown(pts); % normal is pointing negative z
    verticeCount = length(v);
    v = [v;vNew];
    f = [f;fNew+verticeCount];

    if whiteTop
        stlwrite([name, '_blackBase.stl'],f,v);
    else
        stlwrite([name, '_whiteBase.stl'],f,v);
    end

end 

% Top STL ---------------------------------------------
f=[];
v=[];
% top surface: using fan
%  Normal is piont uppositive z. 
bottomEdge = [xValues' zeros(N,1) ones(N,1)*(reliefZ+baseThickness)]; %bottom edge
rightEdge = [xValues(end)*ones(M-1,1) yValues(2:M)' ones(M-1,1)*(reliefZ+baseThickness)];
topEdge = [xValues(N-1:-1:1)', yValues(end)*ones(N-1,1), ones(N-1,1)*(reliefZ+baseThickness)];
leftEdge = [zeros(M-2,1), yValues(M-1:-1:2)', ones(M-2,1)*(reliefZ+baseThickness)];
pts =[bottomEdge; rightEdge; topEdge; leftEdge];

[f,v] = trianglesFan(pts);

for m=1:imageSize(1)-1
    v1 =[];
    v2=[];
    for n=1:imageSize(2)
        v1 = [v1; xValues(n), yValues(m), pixelVals(m,n)+baseThickness];
        v2 = [v2; xValues(n), yValues(m+1), pixelVals(m+1,n)+baseThickness];
    end
    [fNew,vNew] = reverseAddQuadStrip(v1,v2);
     verticeCount = length(v);
     f = [f;fNew+verticeCount];
      v = [v;vNew];
end

%sides
% drawSides:
% bottom edge

for n=1:N-1
        pt4 = [xValues(n), 0, reliefZ+baseThickness];
        pt3 = [xValues(n+1), 0, reliefZ+baseThickness];
        pt2 = [xValues(n+1), 0, pixelVals(1,n+1)+baseThickness];
        pt1 = [xValues(n),0, pixelVals(1,n)+baseThickness];
        [fNew,vNew] = add2Triangles(pt1,pt2,pt3,pt4);
         verticeCount = length(v);
        v = [v;vNew];
        f = [f;fNew+verticeCount];
 end

% right edge
for m=1:M-1
        pt4 = [xValues(end), yValues(m), reliefZ+baseThickness];
        pt3 = [xValues(end), yValues(m+1), reliefZ+baseThickness];
        pt2 = [xValues(end), yValues(m+1), pixelVals(m+1,N)+baseThickness];
        pt1 = [xValues(end),yValues(m), pixelVals(m,N)+baseThickness];
        [fNew,vNew] = add2Triangles(pt1,pt2,pt3,pt4);
        verticeCount = length(v);
        v = [v;vNew];
        f = [f;fNew+verticeCount];
end

%top edge
for n=N:-1:2
   
        pt4 = [xValues(n), yValues(end), reliefZ+baseThickness];
        pt3 = [xValues(n-1), yValues(end), reliefZ+baseThickness];
        pt2 = [xValues(n-1), yValues(end), pixelVals(M,n-1)+baseThickness];
        pt1 = [xValues(n), yValues(end), pixelVals(M,n)+baseThickness];
        [fNew,vNew] = add2Triangles(pt1,pt2,pt3,pt4);
        verticeCount = length(v);
        v = [v;vNew];
        f = [f;fNew+verticeCount];

end

%left edge:
for m=M:-1:2
        pt4 = [0, yValues(m), reliefZ+baseThickness];
        pt3 = [0, yValues(m-1), reliefZ+baseThickness];
        pt2 = [0, yValues(m-1), pixelVals(m-1,1)+baseThickness];
        pt1 = [0, yValues(m), pixelVals(m,1)+baseThickness];
        [fNew,vNew] = add2Triangles(pt1,pt2,pt3,pt4);
        verticeCount = length(v);
        v = [v;vNew];
        f = [f;fNew+verticeCount];

end
if whiteTop
    stlwrite([name, '_whiteTop.stl'],f,v);
else
    stlwrite([name, '_blackTop.stl'],f,v);
end


% ALT BASE: just a box. 
baseHeight = reliefZ; 
v=[];
f=[];
triCount = 0;
verticeCount = 0;

% top surface: 
pt1 = [0, 0, baseHeight];
pt2 = [xValues(N), 0, baseHeight];
pt3 = [xValues(N), yValues(M), baseHeight];
pt4 = [0, yValues(M), baseHeight];
[fNew,vNew] = add2Triangles(pt1,pt2,pt3,pt4);
 verticeCount = length(v);
v = [v;vNew];
f = [f;fNew+verticeCount];


% drawSides:
% bottom edge
pt1 = [0, 0, 0];
pt2 = [xValues(N), 0, 0];
pt3 = [xValues(N), 0,baseHeight];
pt4 = [0, 0, baseHeight];
[fNew,vNew] = add2Triangles(pt1,pt2,pt3,pt4);
 verticeCount = length(v);
v = [v;vNew];
f = [f;fNew+verticeCount];
 

% right edge
        pt1 = [xValues(end), 0, 0];
        pt2 = [xValues(end), yValues(end), 0];
        pt3 = [xValues(end), yValues(end), baseHeight];
        pt4 = [xValues(end),0, baseHeight];
        [fNew,vNew] = add2Triangles(pt1,pt2,pt3,pt4);
        verticeCount = length(v);
        v = [v;vNew];
        f = [f;fNew+verticeCount];

%top edge

pt1 = [xValues(end), yValues(end), 0];
pt2 = [0, yValues(end), 0];
pt3 = [0, yValues(end), baseHeight];
pt4 = [xValues(end), yValues(end), baseHeight];
[fNew,vNew] = add2Triangles(pt1,pt2,pt3,pt4);
verticeCount = length(v);
v = [v;vNew];
f = [f;fNew+verticeCount];

%left edge:

pt1 = [0, yValues(M), 0];
pt2 = [0, 0, 0];
pt3 = [0, 0, baseHeight];
pt4 = [0, yValues(M), baseHeight];
[fNew,vNew] = add2Triangles(pt1,pt2,pt3,pt4);
verticeCount = length(v);
v = [v;vNew];
f = [f;fNew+verticeCount];


% bottom
pt1 = [0, 0, 0];
pt2 = [0, yValues(M), 0];
pt3 = [xValues(N), yValues(M), 0];
pt4 = [xValues(N),0,0];
[fNew,vNew] = add2Triangles(pt1,pt2,pt3,pt4);
verticeCount = length(v);
v = [v;vNew];
f = [f;fNew+verticeCount];


if whiteTop
    stlwrite([name, '_blackBox.stl'],f,v);
else
    stlwrite([name, '_whiteBox.stl'],f,v);
end
toc % return end time